'use strict';
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const through2 = require("through2");
const Vinyl = require("vinyl");
const scanner_1 = require("../services/scanner");
const resolver_1 = require("../providers/resolver");
const paths_1 = require("../utils/paths");
const fs_1 = require("../utils/fs");
class Stream {
    constructor(root, storage, language, options) {
        this.root = root;
        this.storage = storage;
        this.language = language;
        this.options = options;
        // :)
    }
    run(filepath, stats) {
        const scanner = new scanner_1.Scanner(this.root, this.storage, this.language, this.options);
        const resolver = new resolver_1.Resolver(this.storage);
        const _this = this;
        return through2.obj(function (file, enc, cb) {
            let mainFile = '';
            if (file.path) {
                mainFile = paths_1.relative(file.cwd, file.path);
            }
            if (!mainFile.startsWith(_this.root)) {
                mainFile = paths_1.join(_this.root, mainFile);
            }
            // Update Storage
            scanner.scan(filepath, stats).then((lastChangedFile) => {
                // Protection for bad filepath
                if (!filepath && !lastChangedFile) {
                    this.push(file);
                    return cb();
                }
                filepath = filepath ? filepath : lastChangedFile;
                const changedFile = paths_1.normalize(filepath);
                if (resolver.checkDependency(mainFile, changedFile)) {
                    if (_this.options.makeVinylFile) {
                        return _this.makeVinylFile(mainFile).then((vFile) => {
                            this.push(vFile);
                            _this.options.log(mainFile);
                            return cb();
                        });
                    }
                    this.push(file);
                    _this.options.log(mainFile);
                    return cb();
                }
                cb();
            }).catch(cb);
        });
    }
    makeVinylFile(filepath) {
        return __awaiter(this, void 0, void 0, function* () {
            const exists = yield fs_1.pathExists(filepath);
            if (!exists) {
                return null;
            }
            const stat = yield fs_1.statFile(filepath);
            const content = yield fs_1.readFile(filepath);
            return new Vinyl({
                cwd: process.cwd(),
                base: this.root,
                path: filepath,
                contents: new Buffer(content),
                stat
            });
        });
    }
}
exports.Stream = Stream;
